package cn.com.qmth.dp.examcloud.oe.modules.export_course_questions_count;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationOperation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.stereotype.Service;

import cn.com.qmth.dp.examcloud.oe.entity.question.Course;
import cn.com.qmth.dp.examcloud.oe.entity.question.Question;
import cn.com.qmth.dp.examcloud.oe.enums.question.QuesStructType;
import cn.com.qmth.dp.examcloud.oe.multithread.Consumer;

@Service
@Scope("prototype")
public class ExportQuesConsumer extends Consumer<Course> {

	@Autowired
	private MongoTemplate mongoTemplate;

	@Override
	public void consume(Course c) {
		CourseQuestionsCountRetDto rd = new CourseQuestionsCountRetDto();
		addRet(rd);
		Map<String, Object> param=getParam();
		String rootOrgId=(String)param.get("rootOrgId");
		rd.setCourseCode(c.getCode());
		rd.setCourseName(c.getName());
		List<QuestionTypeCount> tc = countByType(c.getCode(),rootOrgId);
		if (CollectionUtils.isNotEmpty(tc)) {
			setCount(rd, tc);
		}

	}
	private void setCount(CourseQuestionsCountRetDto rd, List<QuestionTypeCount> tc) {
		for (QuestionTypeCount t : tc) {
			if (t.getQuestionType().equals(QuesStructType.SINGLE_ANSWER_QUESTION.name())) {
				rd.setSingleAnswerQuestion(t.getCount());
			} else if (t.getQuestionType().equals(QuesStructType.MULTIPLE_ANSWER_QUESTION.name())) {
				rd.setMultipleAnswerQuestion(t.getCount());
			} else if (t.getQuestionType().equals(QuesStructType.BOOL_ANSWER_QUESTION.name())) {
				rd.setBoolAnswerQuestion(t.getCount());
			} else if (t.getQuestionType().equals(QuesStructType.FILL_BLANK_QUESTION.name())) {
				rd.setFillBlankQuestion(t.getCount());
			} else if (t.getQuestionType().equals(QuesStructType.TEXT_ANSWER_QUESTION.name())) {
				rd.setTextAnswerQuestion(t.getCount());
			} else if (t.getQuestionType().equals(QuesStructType.NESTED_ANSWER_QUESTION.name())) {
				rd.setNestedAnswerQuestion(t.getCount());
			}
		}
	}
	private List<QuestionTypeCount> countByType(String courseCode,String rootOrgId) {
		List<AggregationOperation> operations = new ArrayList<>();
		operations.add(Aggregation.match(Criteria.where("orgId").is(rootOrgId).and("course.code").is(courseCode)));
		operations.add(Aggregation.group("questionType").count().as("count"));
		operations.add(Aggregation.project("count").and("questionType").previousOperation());
		Aggregation aggregation = Aggregation.newAggregation(operations);
		AggregationResults<QuestionTypeCount> outputTypeCount = mongoTemplate.aggregate(aggregation, Question.class,
				QuestionTypeCount.class);
		return outputTypeCount.getMappedResults();
	}


}
